@@ -23,7 +23,6 @@
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/filesystem.hpp>
-#include <boost/progress.hpp>
 
 #include <htslib/faidx.h>
 #include <htslib/vcf.h>
@@ -58,6 +57,7 @@ namespace torali
     int32_t nchr;
     int32_t minimumFlankSize;
     int32_t indelsize;
+    int32_t maxInsertionSize;
     uint32_t graphPruning;
     uint32_t minRefSep;
     uint32_t maxReadSep;
@@ -67,7 +67,6 @@ namespace torali
     float flankQuality;
     bool hasExcludeFile;
     bool hasVcfFile;
-    bool isHaplotagged;
     bool hasDumpFile;
     bool svtcmd;
     std::set<int32_t> svtset;
@@ -180,9 +179,9 @@ namespace torali
     
     // Output library statistics
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Sample statistics" << std::endl;
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "Sample statistics" << std::endl;
     for(uint32_t file_c = 0; file_c < c.files.size(); ++file_c) {
-      std::cout << "Sample:" << c.sampleName[file_c] << ",ReadSize=" << sampleLib[file_c].rs << ",Median=" << sampleLib[file_c].median << ",MAD=" << sampleLib[file_c].mad << ",UniqueDiscordantPairs=" << sampleLib[file_c].abnormal_pairs << std::endl;
+      std::cerr << "Sample:" << c.sampleName[file_c] << ",ReadSize=" << sampleLib[file_c].rs << ",Median=" << sampleLib[file_c].median << ",MAD=" << sampleLib[file_c].mad << ",UniqueDiscordantPairs=" << sampleLib[file_c].abnormal_pairs << std::endl;
     }
     
 #ifdef PROFILE
@@ -191,16 +190,15 @@ namespace torali
   
     // End
     now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] Done." << std::endl;;
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] Done." << std::endl;;
     return 0;
   }
 
 
   int delly(int argc, char **argv) {
     Config c;
-    c.isHaplotagged = false;
     c.madNormalCutoff = 5;
-
+    
     // Define generic options
     std::string svtype;
     boost::program_options::options_description generic("Generic options");
@@ -209,7 +207,7 @@ namespace torali
       ("svtype,t", boost::program_options::value<std::string>(&svtype)->default_value("ALL"), "SV type to compute [DEL, INS, DUP, INV, BND, ALL]")
       ("genome,g", boost::program_options::value<boost::filesystem::path>(&c.genome), "genome fasta file")
       ("exclude,x", boost::program_options::value<boost::filesystem::path>(&c.exclude), "file with regions to exclude")
-      ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile)->default_value("sv.bcf"), "SV BCF output file")
+      ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile), "BCF output file")
       ;
     
     boost::program_options::options_description disc("Discovery options");
@@ -249,16 +247,15 @@ namespace torali
     boost::program_options::variables_map vm;
     boost::program_options::store(boost::program_options::command_line_parser(argc, argv).options(cmdline_options).positional(pos_args).run(), vm);
     boost::program_options::notify(vm);
-    
 
     // Check command line arguments
     if ((vm.count("help")) || (!vm.count("input-file")) || (!vm.count("genome"))) { 
-      std::cout << std::endl;
-      std::cout << "Usage: delly " << argv[0] << " [OPTIONS] -g <ref.fa> <sample1.sort.bam> <sample2.sort.bam> ..." << std::endl;
-      std::cout << visible_options << "\n";
+      std::cerr << std::endl;
+      std::cerr << "Usage: delly " << argv[0] << " [OPTIONS] -g <ref.fa> <sample1.sort.bam> <sample2.sort.bam> ..." << std::endl;
+      std::cerr << visible_options << "\n";
       return 0;
     }
-    
+
     // SV types to compute?
     _svTypesToCompute(c, svtype, vm.count("svtype"));
     
@@ -333,6 +330,7 @@ namespace torali
       hts_idx_destroy(idx);
       sam_close(samfile);
     }
+    checkSampleNames(c);
     
     // Check exclude file
     if (vm.count("exclude")) {
@@ -364,15 +362,20 @@ namespace torali
       c.hasVcfFile = true;
     } else c.hasVcfFile = false;
     
-    // Check output directory
-    if (!_outfileValid(c.outfile)) return 1;
+    // Check outfile
+    if (!vm.count("outfile")) c.outfile = "-";
+    else {
+      if (c.outfile.string() != "-") {
+	if (!_outfileValid(c.outfile)) return 1;
+      }
+    }
     
     // Show cmd
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] ";
-    std::cout << "delly ";
-    for(int i=0; i<argc; ++i) { std::cout << argv[i] << ' '; }
-    std::cout << std::endl;
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] ";
+    std::cerr << "delly ";
+    for(int i=0; i<argc; ++i) { std::cerr << argv[i] << ' '; }
+    std::cerr << std::endl;
     
     // Always ignore reads of mapping quality <5 for genotyping, otherwise het. is more likely!
     if (c.minGenoQual<5) c.minGenoQual=5;
@@ -381,7 +384,8 @@ namespace torali
     c.aliscore = DnaScore<int>(5, -4, -10, -1);
     c.flankQuality = 0.95;
     c.minimumFlankSize = 13;
-    c.indelsize = 500;
+    c.indelsize = 1000;
+    c.maxInsertionSize = 200;
     return dellyRun(c);
   }
 
