@@ -9,7 +9,6 @@
 #include <boost/iostreams/copy.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
 #include <boost/iostreams/device/file.hpp>
-#include <boost/progress.hpp>
 
 #include <htslib/sam.h>
 
@@ -85,21 +84,47 @@ namespace torali
     }
   };
 
+
+  inline int32_t
+  _selectReadStart(std::vector<Junction> const& jcvec) {
+    for(uint32_t i = 0; i < jcvec.size(); ++i) {
+      if (jcvec[i].rstart != -1) return jcvec[i].rstart;
+    }
+    return -1;
+  }  
+  
   // Deletion junctions
   template<typename TConfig, typename TReadBp>
   inline void
   selectDeletions(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
       if (it->second.size() > 1) {
+	int32_t rst = _selectReadStart(it->second);
 	for(uint32_t i = 0; i < it->second.size(); ++i) {
 	  for(uint32_t j = i+1; j < it->second.size(); ++j) {
 	    if ((uint32_t) (it->second[j].seqpos - it->second[i].seqpos) > c.maxReadSep) break;
 	    // Same chr, same direction, opposing soft-clips
 	    if ((it->second[j].refidx == it->second[i].refidx) && (it->second[j].forward == it->second[i].forward) && (it->second[i].scleft != it->second[j].scleft)) {
 	      // Min. deletion size
-	      if ( (uint32_t) std::abs(it->second[j].refpos - it->second[i].refpos) > c.minRefSep) {
-		int32_t rst = it->second[i].rstart;
-		if (rst == -1) rst = it->second[j].rstart;
+	      int32_t dellen = 0;
+	      if (it->second[i].forward) {
+		if (!it->second[i].scleft) {
+		  if (it->second[i].refpos <= it->second[j].refpos) dellen = (it->second[j].refpos - it->second[i].refpos) - (it->second[j].seqpos - it->second[i].seqpos);
+		  else dellen = 0;
+		} else {
+		  if (it->second[i].refpos <= it->second[j].refpos) dellen = 0;
+		  else dellen = (it->second[i].refpos - it->second[j].refpos) + (it->second[j].seqpos - it->second[i].seqpos);
+		}
+	      } else {
+		if (it->second[i].scleft) {
+		  if (it->second[i].refpos <= it->second[j].refpos) dellen = 0;
+		  else dellen = (it->second[i].refpos - it->second[j].refpos) - (it->second[j].seqpos - it->second[i].seqpos);
+		} else {
+		  if (it->second[i].refpos <= it->second[j].refpos) dellen = (it->second[j].refpos - it->second[i].refpos) + (it->second[j].seqpos - it->second[i].seqpos); 
+		  else dellen = 0;
+		}
+	      }
+	      if (dellen > (int32_t) c.minRefSep) {
 		// Avg. qval
 		int32_t qval = (int32_t) (((int32_t) it->second[i].qual + (int32_t) it->second[j].qual) / 2);
 		// Correct clipping architecture, note: soft-clipping of error-prone reads can lead to switching left/right breakpoints
@@ -112,6 +137,7 @@ namespace torali
 		    br[2].push_back(SRBamRecord(it->second[j].refidx, it->second[j].refpos, it->second[i].refidx, it->second[i].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
 		  }
 		}
+		break; // Only report first SV split to avoid ambiguities
 	      }
 	    }
 	  }
@@ -127,6 +153,7 @@ namespace torali
   selectDuplications(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
       if (it->second.size() > 1) {
+	int32_t rst = _selectReadStart(it->second);
 	for(uint32_t i = 0; i < it->second.size(); ++i) {
 	  for(uint32_t j = i+1; j < it->second.size(); ++j) {
 	    if ((uint32_t) (it->second[j].seqpos - it->second[i].seqpos) > c.maxReadSep) break;
@@ -134,8 +161,6 @@ namespace torali
 	    if ((it->second[j].refidx == it->second[i].refidx) && (it->second[j].forward == it->second[i].forward) && (it->second[i].scleft != it->second[j].scleft)) {
 	      // Min. duplication size
 	      if ( (uint32_t) std::abs(it->second[j].refpos - it->second[i].refpos) > c.minRefSep) {
-		int32_t rst = it->second[i].rstart;
-		if (rst == -1) rst = it->second[j].rstart;
 		// Avg. qval
 		int32_t qval = (int32_t) (((int32_t) it->second[i].qual + (int32_t) it->second[j].qual) / 2);
 		// Correct clipping architecture, note: soft-clipping of error-prone reads can lead to switching left/right breakpoints
@@ -162,6 +187,7 @@ namespace torali
   selectInversions(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
       if (it->second.size() > 1) {
+	int32_t rst = _selectReadStart(it->second);
 	for(uint32_t i = 0; i < it->second.size(); ++i) {
 	  for(uint32_t j = i+1; j < it->second.size(); ++j) {
 	    if ((uint32_t) (it->second[j].seqpos - it->second[i].seqpos) > c.maxReadSep) break;
@@ -169,8 +195,6 @@ namespace torali
 	    if ((it->second[j].refidx == it->second[i].refidx) && (it->second[j].forward != it->second[i].forward) && (it->second[i].scleft == it->second[j].scleft)) {
 	      // Min. inversion size
 	      if ( (uint32_t) std::abs(it->second[j].refpos - it->second[i].refpos) > c.minRefSep) {
-		int32_t rst = it->second[i].rstart;
-		if (rst == -1) rst = it->second[j].rstart;
 		// Avg. qval
 		int32_t qval = (int32_t) (((int32_t) it->second[i].qual + (int32_t) it->second[j].qual) / 2);
 		if (it->second[i].refpos <= it->second[j].refpos) {
@@ -196,6 +220,7 @@ namespace torali
   selectInsertions(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
       if (it->second.size() > 1) {
+	int32_t rst = _selectReadStart(it->second);
 	for(uint32_t i = 0; i < it->second.size(); ++i) {
 	  for(uint32_t j = i+1; j < it->second.size(); ++j) {
 	    // Same chr, same direction, opposing soft-clips
@@ -203,16 +228,27 @@ namespace torali
 	      // Reference insertion footprint should be small
 	      if ( (uint32_t) std::abs(it->second[j].refpos - it->second[i].refpos) < c.maxReadSep) {
 		// Large separation in sequence space
-		if ((uint32_t) (it->second[j].seqpos - it->second[i].seqpos) > c.minRefSep) {
-		  int32_t rst = it->second[i].rstart;
-		  if (rst == -1) rst = it->second[j].rstart;
+		int32_t isizelen = 0;
+		if (it->second[i].forward) {
+		  if (!it->second[i].scleft) {
+		    if (it->second[i].refpos <= it->second[j].refpos) isizelen = (it->second[j].seqpos - it->second[i].seqpos) - (it->second[j].refpos - it->second[i].refpos);
+		    else isizelen = (it->second[j].seqpos - it->second[i].seqpos) + (it->second[i].refpos - it->second[j].refpos);
+		  } else isizelen = 0;
+		} else {
+		  if (it->second[i].scleft) {
+		    if (it->second[i].refpos <= it->second[j].refpos) isizelen = (it->second[j].seqpos - it->second[i].seqpos) + (it->second[j].refpos - it->second[i].refpos);
+		    else isizelen = (it->second[j].seqpos - it->second[i].seqpos) - (it->second[i].refpos - it->second[j].refpos);
+		  } else isizelen = 0;
+		}
+		if ((isizelen > (int32_t) c.minRefSep) && (isizelen < (int32_t) c.maxInsertionSize)) {
 		  // Avg. qval
 		  int32_t qval = (int32_t) (((int32_t) it->second[i].qual + (int32_t) it->second[j].qual) / 2);
 		  if (it->second[i].refpos <= it->second[j].refpos) {
-		    br[4].push_back(SRBamRecord(it->second[i].refidx, it->second[i].refpos, it->second[j].refidx, it->second[j].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
+		    br[4].push_back(SRBamRecord(it->second[i].refidx, it->second[i].refpos, it->second[j].refidx, it->second[j].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, isizelen, it->first));
 		  } else {
-		    br[4].push_back(SRBamRecord(it->second[j].refidx, it->second[j].refpos, it->second[i].refidx, it->second[i].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
+		    br[4].push_back(SRBamRecord(it->second[j].refidx, it->second[j].refpos, it->second[i].refidx, it->second[i].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, isizelen, it->first));
 		  }
+		  break; // Only report first SV split to avoid ambiguities
 		}
 	      }
 	    }
@@ -229,6 +265,7 @@ namespace torali
   selectTranslocations(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
       if (it->second.size() > 1) {
+	int32_t rst = _selectReadStart(it->second);
 	for(uint32_t i = 0; i < it->second.size(); ++i) {
 	  for(uint32_t j = i+1; j < it->second.size(); ++j) {
 	    if ((uint32_t) (it->second[j].seqpos - it->second[i].seqpos) > c.maxReadSep) break;
@@ -240,18 +277,16 @@ namespace torali
 		chr1ev = i;
 		chr2ev = j;
 	      }
-	      int32_t rst = it->second[i].rstart;
-	      if (rst == -1) rst = it->second[j].rstart;
 	      // Avg. qval
 	      int32_t qval = (int32_t) (((int32_t) it->second[i].qual + (int32_t) it->second[j].qual) / 2);
 	      if (it->second[chr1ev].forward == it->second[chr2ev].forward) {
 		// Same direction, opposing soft-clips
 		if (it->second[chr1ev].scleft != it->second[chr2ev].scleft) {
 		  if (it->second[chr1ev].scleft) {
-		    // 5to3
+		    // 3to5
 		    br[DELLY_SVT_TRANS + 2].push_back(SRBamRecord(it->second[chr2ev].refidx, it->second[chr2ev].refpos, it->second[chr1ev].refidx, it->second[chr1ev].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
 		  } else {
-		    // 3to5
+		    // 5to3
 		    br[DELLY_SVT_TRANS + 3].push_back(SRBamRecord(it->second[chr2ev].refidx, it->second[chr2ev].refpos, it->second[chr1ev].refidx, it->second[chr1ev].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
 		  }
 		}
@@ -259,10 +294,10 @@ namespace torali
 		// Opposing direction, same soft-clips
 		if (it->second[chr1ev].scleft == it->second[chr2ev].scleft) {
 		  if (it->second[chr1ev].scleft) {
-		    // 3to3
+		    // 5to5
 		    br[DELLY_SVT_TRANS + 1].push_back(SRBamRecord(it->second[chr2ev].refidx, it->second[chr2ev].refpos, it->second[chr1ev].refidx, it->second[chr1ev].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
 		  } else {
-		    // 5to5
+		    // 3to3
 		    br[DELLY_SVT_TRANS + 0].push_back(SRBamRecord(it->second[chr2ev].refidx, it->second[chr2ev].refpos, it->second[chr1ev].refidx, it->second[chr1ev].refpos, rst, std::min(it->second[j].seqpos, it->second[i].seqpos), qval, std::abs(it->second[j].seqpos - it->second[i].seqpos), it->first));
 		  }
 		}
@@ -294,12 +329,10 @@ namespace torali
     
     // Parse genome chr-by-chr
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Split-read scanning" << std::endl;
-    boost::progress_display show_progress( hdr->n_targets );
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "Split-read scanning" << std::endl;
 
     // Iterate chromosomes
     for(int32_t refIndex=0; refIndex < (int32_t) hdr->n_targets; ++refIndex) {
-      ++show_progress;
       if (validRegions[refIndex].empty()) continue;
 
       // Collect reads from all samples
@@ -427,10 +460,10 @@ namespace torali
   fetchSVs(TConfig const& c, TReadBp& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     // Extract BAM records
     if ((!c.svtcmd) || (c.svtset.find(2) != c.svtset.end())) selectDeletions(c, readBp, br);
-    //if ((!c.svtcmd) || (c.svtset.find(3) != c.svtset.end())) selectDuplications(c, readBp, br);
-    //if ((!c.svtcmd) || (c.svtset.find(0) != c.svtset.end()) || (c.svtset.find(1) != c.svtset.end())) selectInversions(c, readBp, br);
+    if ((!c.svtcmd) || (c.svtset.find(3) != c.svtset.end())) selectDuplications(c, readBp, br);
+    if ((!c.svtcmd) || (c.svtset.find(0) != c.svtset.end()) || (c.svtset.find(1) != c.svtset.end())) selectInversions(c, readBp, br);
     if ((!c.svtcmd) || (c.svtset.find(4) != c.svtset.end())) selectInsertions(c, readBp, br);
-    //if ((!c.svtcmd) || (c.svtset.find(5) != c.svtset.end()) || (c.svtset.find(6) != c.svtset.end()) || (c.svtset.find(7) != c.svtset.end()) || (c.svtset.find(8) != c.svtset.end())) selectTranslocations(c, readBp, br);
+    if ((!c.svtcmd) || (c.svtset.find(5) != c.svtset.end()) || (c.svtset.find(6) != c.svtset.end()) || (c.svtset.find(7) != c.svtset.end()) || (c.svtset.find(8) != c.svtset.end())) selectTranslocations(c, readBp, br);
   }
 
   template<typename TConfig, typename TValidRegions, typename TSvtSRBamRecord>
@@ -448,7 +481,6 @@ namespace torali
   template<typename TConfig, typename TValidRegions, typename TSVs, typename TSRStore>
   inline void
   _clusterSRReads(TConfig const& c, TValidRegions const& validRegions, TSVs& svc, TSRStore& srStore) {
-    typedef typename TSRStore::mapped_type TSvPosVector;
     // Split-reads
     typedef std::vector<SRBamRecord> TSRBamRecord;
     typedef std::vector<TSRBamRecord> TSvtSRBamRecord;
@@ -457,7 +489,7 @@ namespace torali
 	 	 
     // Debug
     //outputSRBamRecords(c, srBR);
-    
+
     // Cluster BAM records
     for(uint32_t svt = 0; svt < srBR.size(); ++svt) {
       if (srBR[svt].empty()) continue;
@@ -470,15 +502,27 @@ namespace torali
 
       // Debug
       //outputStructuralVariants(c, svc, srBR, svt);
-      
       // Track split-reads
+      samFile* samfile = sam_open(c.files[0].string().c_str(), "r");
+      bam_hdr_t* hdr = sam_hdr_read(samfile);
       for(uint32_t i = 0; i < srBR[svt].size(); ++i) {
 	// Read assigned?
-	if (srBR[svt][i].svid != -1) {
-	  if (srStore.find(srBR[svt][i].id) == srStore.end()) srStore.insert(std::make_pair(srBR[svt][i].id, TSvPosVector()));
-	  srStore[srBR[svt][i].id].push_back(SeqSlice(srBR[svt][i].svid, srBR[svt][i].sstart, srBR[svt][i].inslen, srBR[svt][i].qual));
+	if ((srBR[svt][i].svid != -1) && (srBR[svt][i].rstart != -1)) {
+	  if (srBR[svt][i].rstart < (int32_t) hdr->target_len[srBR[svt][i].chr]) {
+	    if (srStore[srBR[svt][i].chr].find(std::make_pair(srBR[svt][i].rstart, srBR[svt][i].id)) == srStore[srBR[svt][i].chr].end()) srStore[srBR[svt][i].chr].insert(std::make_pair(std::make_pair(srBR[svt][i].rstart, srBR[svt][i].id), std::vector<SeqSlice>()));
+	    srStore[srBR[svt][i].chr][std::make_pair(srBR[svt][i].rstart, srBR[svt][i].id)].push_back(SeqSlice(srBR[svt][i].svid, srBR[svt][i].sstart, srBR[svt][i].inslen, srBR[svt][i].qual));
+	  }
+	  if (srBR[svt][i].chr != srBR[svt][i].chr2) {
+	    // Unclear which chr was primary alignment so insert both if and only if rstart < reference length
+	    if (srBR[svt][i].rstart < (int32_t) hdr->target_len[srBR[svt][i].chr2]) {
+	      if (srStore[srBR[svt][i].chr2].find(std::make_pair(srBR[svt][i].rstart, srBR[svt][i].id)) == srStore[srBR[svt][i].chr2].end()) srStore[srBR[svt][i].chr2].insert(std::make_pair(std::make_pair(srBR[svt][i].rstart, srBR[svt][i].id), std::vector<SeqSlice>()));
+	      srStore[srBR[svt][i].chr2][std::make_pair(srBR[svt][i].rstart, srBR[svt][i].id)].push_back(SeqSlice(srBR[svt][i].svid, srBR[svt][i].sstart, srBR[svt][i].inslen, srBR[svt][i].qual));
+	    }
+	  }
 	}
       }
+      bam_hdr_destroy(hdr);
+      sam_close(samfile);
     }
   }
 
@@ -491,12 +535,12 @@ namespace torali
     bam_hdr_t* hdr = sam_hdr_read(samfile);
 
     // Header
-    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tinslen" << std::endl;
+    std::cerr << "id\tchr1\tpos1\tchr2\tpos2\tsvtype\tct\tinslen" << std::endl;
 
     // SVs
     for(uint32_t svt = 0; svt < br.size(); ++svt) {
       for(uint32_t i = 0; i < br[svt].size(); ++i) {
-	std::cerr << hdr->target_name[br[svt][i].chr] << '\t' << br[svt][i].pos << '\t' << hdr->target_name[br[svt][i].chr2] << '\t' << br[svt][i].pos2 << '\t' << _addID(svt) << '\t' << _addOrientation(svt) << '\t' << br[svt][i].inslen << std::endl;
+	std::cerr << br[svt][i].id << '\t' << hdr->target_name[br[svt][i].chr] << '\t' << br[svt][i].pos << '\t' << hdr->target_name[br[svt][i].chr2] << '\t' << br[svt][i].pos2 << '\t' << _addID(svt) << '\t' << _addOrientation(svt) << '\t' << br[svt][i].inslen << std::endl;
       }
     }
     // Clean-up
@@ -528,7 +572,7 @@ namespace torali
 	hts_itr_t* iter = sam_itr_queryi(idx[file_c], refIndex, 0, hdr->target_len[refIndex]);
 	bam1_t* rec = bam_init1();
 	while (sam_itr_next(samfile[file_c], iter, rec) >= 0) {
-	  if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP | BAM_FSECONDARY | BAM_FSUPPLEMENTARY)) continue;
+	  if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP)) continue;
 	  std::size_t seed = hash_lr(rec);
 	  std::string qname = bam_get_qname(rec);
 	  if (hm.find(seed) == hm.end()) hm.insert(std::make_pair(seed, qname));
@@ -578,18 +622,24 @@ namespace torali
     bam_hdr_t* hdr = sam_hdr_read(samfile);
 
     // Header
-    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tpeSupport\tsrSupport" << std::endl;
+    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tpeSupport\tsrSupport\tconsensus" << std::endl;
     
     // SVs
     for(uint32_t i = 0; i < svs.size(); ++i) {
       if (svs[i].svt != svt) continue;
-      std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << std::endl;
+      std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << '\t' << svs[i].consensus << std::endl;
     }
     
     // Clean-up
     bam_hdr_destroy(hdr);
     sam_close(samfile);
   }
+
+  template<typename TConfig>
+  inline void
+  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs) {
+    for(uint32_t i = 0; i <= 8; ++i) outputStructuralVariants(c, svs, i);
+  }
   
 
 }
