@@ -21,6 +21,7 @@
 #include <htslib/vcf.h>
 #include <htslib/faidx.h>
 
+#include "edlib.h"
 #include "delly.h"
 #include "coverage.h"
 #include "genotype.h"
@@ -34,10 +35,9 @@ namespace torali {
 
 
   struct TeguaConfig {
-    bool islr;
     bool hasDumpFile;
     bool hasExcludeFile;
-    bool isHaplotagged;
+    bool hasVcfFile;
     bool svtcmd;
     uint16_t minMapQual;
     uint16_t minGenoQual;
@@ -49,12 +49,15 @@ namespace torali {
     uint32_t maxReadPerSV;
     int32_t nchr;
     int32_t minimumFlankSize;
+    int32_t indelsize;
+    int32_t maxInsertionSize;
     float indelExtension;
     float flankQuality;
     std::set<int32_t> svtset;
     DnaScore<int> aliscore;
     boost::filesystem::path dumpfile;
     boost::filesystem::path outfile;
+    boost::filesystem::path vcffile;
     std::vector<boost::filesystem::path> files;
     boost::filesystem::path genome;
     boost::filesystem::path exclude;
@@ -106,26 +109,25 @@ namespace torali {
      return 1;
    }
      
-   // SR Store
-   typedef std::vector<SeqSlice> TSvPosVector;
-   typedef boost::unordered_map<std::size_t, TSvPosVector> TReadSV;
-   TReadSV srStore;
-
-   // Identify SVs
-   if (srStore.empty()) {
+   // SV Discovery
+   if (!c.hasVcfFile) {
        
      // Structural Variant Candidates
      typedef std::vector<StructuralVariantRecord> TVariants;
      TVariants svc;
 
-     // Temporary split-read store
-     TReadSV tmpStore;
+     // Split-read store
+     typedef std::pair<int32_t, std::size_t> TPosRead;
+     typedef std::vector<SeqSlice> TSvPosVector;
+     typedef boost::unordered_map<TPosRead, TSvPosVector> TPosReadSV;
+     typedef std::vector<TPosReadSV> TGenomicPosReadSV;
+     TGenomicPosReadSV srStore(c.nchr, TPosReadSV());
 
      // SV Discovery
-     _clusterSRReads(c, validRegions, svc, tmpStore);
+     _clusterSRReads(c, validRegions, svc, srStore);
 
      // Assemble
-     assemble(c, validRegions, svc, tmpStore);
+     assemble(c, validRegions, svc, srStore);
 
      // Sort SVs
      sort(svc.begin(), svc.end(), SortSVs<StructuralVariantRecord>());
@@ -136,7 +138,7 @@ namespace torali {
        if ((svIter->srSupport == 0) && (svIter->peSupport == 0)) continue;
        // Duplicate?
        if (!svs.empty()) {
-	 if ((lastSV.chr == svIter->chr) && (lastSV.chr2 == svIter->chr2) && (std::abs(svIter->svStart - lastSV.svStart) < c.minRefSep) && (std::abs(svIter->svEnd - lastSV.svEnd) < c.minRefSep)) continue;
+	 if ((lastSV.chr == svIter->chr) && (lastSV.chr2 == svIter->chr2) && (lastSV.svt == svIter->svt) && (std::abs(svIter->svStart - lastSV.svStart) < c.minRefSep) && (std::abs(svIter->svEnd - lastSV.svEnd) < c.minRefSep)) continue;
        }
        lastSV = *svIter;
        svs.push_back(*svIter);
@@ -145,27 +147,11 @@ namespace torali {
      // Sort
      sort(svs.begin(), svs.end(), SortSVs<StructuralVariantRecord>());
      
-     // Re-number SVs and update SR Store
-     typedef std::map<uint32_t, uint32_t> TIdMap;
-     TIdMap idmap;
+     // Re-number SVs
      uint32_t cliqueCount = 0;
-     for(typename TVariants::iterator svIt = svs.begin(); svIt != svs.end(); ++svIt, ++cliqueCount) {
-       idmap.insert(std::make_pair(svIt->id, cliqueCount));
-       svIt->id = cliqueCount;
-     }
-     for(typename TReadSV::iterator ts = tmpStore.begin(); ts != tmpStore.end(); ++ts) {
-       bool keep = false;
-       for(uint32_t idx = 0; idx < ts->second.size(); ++idx) {
-	 if (idmap.find(ts->second[idx].svid) == idmap.end()) ts->second[idx].svid = -1;
-	 else {
-	   ts->second[idx].svid = idmap.find(ts->second[idx].svid)->second;
-	   keep = true;
-	 }
-       }
-       if (keep) srStore.insert(*ts);
-     }
+     for(typename TVariants::iterator svIt = svs.begin(); svIt != svs.end(); ++svIt, ++cliqueCount) svIt->id = cliqueCount;
      //outputStructuralVariants(c, svs);
-   }
+   } else vcfParse(c, hdr, svs);
    // Clean-up
    bam_hdr_destroy(hdr);
    sam_close(samfile);
@@ -192,9 +178,8 @@ namespace torali {
      rcMap[file_c].resize(svs.size(), ReadCount());
    }
       
-   // Reference SV Genotyping
-   //trackRef(c, svs, jctMap, rcMap);
-   genotypeLR(c, svs, srStore, jctMap, rcMap);
+   // SV Genotyping
+   genotypeLR(c, svs, jctMap, rcMap);
 
    // VCF Output
    vcfOutput(c, svs, jctMap, rcMap, spanMap);
@@ -205,15 +190,13 @@ namespace torali {
 
    // End
    boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-   std::cout << '[' << boost::posix_time::to_simple_string(now) << "] Done." << std::endl;;
+   std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] Done." << std::endl;;
   
    return 0;
  }
 
  int tegua(int argc, char **argv) {
    TeguaConfig c;
-   c.isHaplotagged = false;
-   c.islr = true;
    
    // Parameter
    std::string svtype;
@@ -226,27 +209,30 @@ namespace torali {
      ("technology,y", boost::program_options::value<std::string>(&mode)->default_value("ont"), "seq. technology [pb, ont]")
      ("genome,g", boost::program_options::value<boost::filesystem::path>(&c.genome), "genome fasta file")
      ("exclude,x", boost::program_options::value<boost::filesystem::path>(&c.exclude), "file with regions to exclude")
-     ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile)->default_value("sv.bcf"), "SV BCF output file")
+     ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile), "BCF output file")
      ;
    
    boost::program_options::options_description disc("Discovery options");
    disc.add_options()
      ("mapqual,q", boost::program_options::value<uint16_t>(&c.minMapQual)->default_value(10), "min. mapping quality")
      ("minclip,c", boost::program_options::value<uint32_t>(&c.minClip)->default_value(25), "min. clipping length")
-     ("min-clique-size,z", boost::program_options::value<uint32_t>(&c.minCliqueSize)->default_value(2), "min. clique size")     
+     ("min-clique-size,z", boost::program_options::value<uint32_t>(&c.minCliqueSize)->default_value(3), "min. clique size")     
      ("minrefsep,m", boost::program_options::value<uint32_t>(&c.minRefSep)->default_value(30), "min. reference separation")
-     ("maxreadsep,n", boost::program_options::value<uint32_t>(&c.maxReadSep)->default_value(75), "max. read separation")
+     ("maxreadsep,n", boost::program_options::value<uint32_t>(&c.maxReadSep)->default_value(100), "max. read separation")
      ;
 
    boost::program_options::options_description cons("Consensus options");
    cons.add_options()
-     ("max-reads,p", boost::program_options::value<uint32_t>(&c.maxReadPerSV)->default_value(5), "max. reads for consensus computation")
-     ("flank-size,f", boost::program_options::value<int32_t>(&c.minimumFlankSize)->default_value(400), "min. flank size")
+     ("max-reads,p", boost::program_options::value<uint32_t>(&c.maxReadPerSV)->default_value(15), "max. reads for consensus computation")
+     ("flank-size,f", boost::program_options::value<int32_t>(&c.minimumFlankSize)->default_value(100), "min. flank size")
      ("flank-quality,a", boost::program_options::value<float>(&c.flankQuality)->default_value(0.9), "min. flank quality")
+     ("indel-size,i", boost::program_options::value<int32_t>(&c.indelsize)->default_value(10000), "use exact alleles for InDels <10kbp")
+     ("max-isize,r", boost::program_options::value<int32_t>(&c.maxInsertionSize)->default_value(10000), "max. insertion size")
      ;     
    
    boost::program_options::options_description geno("Genotyping options");
    geno.add_options()
+     ("vcffile,v", boost::program_options::value<boost::filesystem::path>(&c.vcffile), "input VCF/BCF file for genotyping")
      ("geno-qual,u", boost::program_options::value<uint16_t>(&c.minGenoQual)->default_value(5), "min. mapping quality for genotyping")
      ("dump,d", boost::program_options::value<boost::filesystem::path>(&c.dumpfile), "gzipped output file for SV-reads")
      ;
@@ -272,9 +258,9 @@ namespace torali {
    
    // Check command line arguments
    if ((vm.count("help")) || (!vm.count("input-file")) || (!vm.count("genome"))) {
-     std::cout << std::endl;
-     std::cout << "Usage: delly " << argv[0] << " [OPTIONS] -g <ref.fa> <sample1.sort.bam> <sample2.sort.bam> ..." << std::endl;
-     std::cout << visible_options << "\n";
+     std::cerr << std::endl;
+     std::cerr << "Usage: delly " << argv[0] << " [OPTIONS] -g <ref.fa> <sample1.sort.bam> <sample2.sort.bam> ..." << std::endl;
+     std::cerr << visible_options << "\n";
      return 0;
    }
 
@@ -352,6 +338,7 @@ namespace torali {
      hts_idx_destroy(idx);
      sam_close(samfile);
    }
+   checkSampleNames(c);
 
    // Check exclude file
    if (vm.count("exclude")) {
@@ -361,16 +348,42 @@ namespace torali {
      }
      c.hasExcludeFile = true;
    } else c.hasExcludeFile = false;
-   
-   // Check output directory
-   if (!_outfileValid(c.outfile)) return 1;
+
+   // Check input VCF file
+   if (vm.count("vcffile")) {
+     if (!(boost::filesystem::exists(c.vcffile) && boost::filesystem::is_regular_file(c.vcffile) && boost::filesystem::file_size(c.vcffile))) {
+       std::cerr << "Input VCF/BCF file is missing: " << c.vcffile.string() << std::endl;
+       return 1;
+     }
+     htsFile* ifile = bcf_open(c.vcffile.string().c_str(), "r");
+     if (ifile == NULL) {
+       std::cerr << "Fail to open file " << c.vcffile.string() << std::endl;
+       return 1;
+     }
+     bcf_hdr_t* hdr = bcf_hdr_read(ifile);
+     if (hdr == NULL) {
+       std::cerr << "Fail to open index file " << c.vcffile.string() << std::endl;
+       return 1;
+     }
+     bcf_hdr_destroy(hdr);
+     bcf_close(ifile);
+     c.hasVcfFile = true;
+   } else c.hasVcfFile = false;
+
+   // Check outfile
+   if (!vm.count("outfile")) c.outfile = "-";
+   else {
+     if (c.outfile.string() != "-") {
+       if (!_outfileValid(c.outfile)) return 1;
+     }
+   }
 
    // Show cmd
    boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-   std::cout << '[' << boost::posix_time::to_simple_string(now) << "] ";
-   std::cout << "delly ";
-   for(int i=0; i<argc; ++i) { std::cout << argv[i] << ' '; }
-   std::cout << std::endl;
+   std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] ";
+   std::cerr << "delly ";
+   for(int i=0; i<argc; ++i) { std::cerr << argv[i] << ' '; }
+   std::cerr << std::endl;
    
    // Run Tegua
    if (mode == "pb") c.indelExtension = 0.7;
