@@ -77,6 +77,21 @@ namespace torali
   };
 
 
+  template<typename TConfig>
+  inline void
+  checkSampleNames(TConfig& c) {
+    uint32_t ucount = 0;
+    std::set<std::string> snames;
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      while (snames.find(c.sampleName[file_c]) != snames.end()) {
+	std::cerr << "Warning: Duplicate sample names: " << c.sampleName[file_c] << std::endl;
+	c.sampleName[file_c] += "_" + boost::lexical_cast<std::string>(ucount++);
+	std::cerr << "Warning: Changing sample name to " << c.sampleName[file_c] << std::endl;
+      }
+      snames.insert(c.sampleName[file_c]);
+    }
+  }
+
   inline bool
   nContent(std::string const& s) {
     for(uint32_t i = 0; i < s.size(); ++i) {
@@ -260,17 +275,6 @@ namespace torali
   lastAlignedPosition(bam1_t const* rec) {
     return rec->core.pos + alignmentLength(rec);
   }
-
-
-
-  inline std::size_t hash_lr(bam1_t* rec) {
-    boost::hash<std::string> string_hash;
-    std::string qname = bam_get_qname(rec);
-    std::size_t seed = hash_string(qname.c_str());
-    boost::hash_combine(seed, string_hash(qname));
-    return seed;
-  }
-      
   
   inline std::size_t hash_pair(bam1_t* rec) {
     std::size_t seed = hash_string(bam_get_qname(rec));
@@ -290,6 +294,14 @@ namespace torali
     return seed;
   }
 
+  inline std::size_t hash_lr(bam1_t* rec) {
+    boost::hash<std::string> string_hash;
+    std::string qname = bam_get_qname(rec);
+    std::size_t seed = hash_string(qname.c_str());
+    boost::hash_combine(seed, string_hash(qname));
+    return seed;
+  }
+  
   inline void
   reverseComplement(std::string& sequence) {
     std::string rev = boost::to_upper_copy(std::string(sequence.rbegin(), sequence.rend()));
@@ -621,7 +633,7 @@ namespace torali
 	      if ((rec->core.flag & BAM_FPAIRED) && !(rec->core.flag & BAM_FMUNMAP) && (rec->core.tid==rec->core.mtid)) {
 		if (processedNumPairs < maxNumAlignments) {
 		  vecISize.push_back(abs(rec->core.isize));
-		  if (getSVType(rec->core) == 2) ++rplus;
+		  if (getSVType(rec) == 2) ++rplus;
 		  else ++nonrplus;
 		  ++processedNumPairs;
 		}
@@ -682,31 +694,6 @@ namespace torali
   }
 
 
-  template<typename TAlign>
-  inline uint32_t
-  _trimAlignedSequences(TAlign const& align, std::string& s0, std::string& s1) {
-    int32_t s = -1;
-    int32_t e = -1;
-    uint32_t leadCrop = 0;
-    for(uint32_t j = 0; j<align.shape()[1]; ++j) {
-      if (align[0][j] != '-') {
-	if (align[1][j] != '-') {
-	  if (s == -1) s = j;
-	  e = j + 1;
-	}
-	if (s == -1) ++leadCrop;
-      }
-    }
-    s0.clear();
-    s1.clear();
-    for(int32_t j = s; j < e; ++j) {
-      if (align[0][j] != '-') s0.push_back(align[0][j]);
-      if (align[1][j] != '-') s1.push_back(align[1][j]);
-    }
-    return leadCrop;
-  }
-
-
 }
 
 #endif
